@@ -30,6 +30,7 @@ class Trainer(nn.Module):
         train_chromosome_ids,
         valid_chromosome_ids,
         batch_size,
+        context_length,
         lr = 3e-4,
         wd = 0.1,
         validate_every = 250,
@@ -37,7 +38,6 @@ class Trainer(nn.Module):
         grad_accum_every = 1,
         held_out_targets = [],
         held_out_cell_types = [],
-        context_length = 4096,
         shuffle = False,
         train_sample_frac = 1.,
         valid_sample_frac = 1.,
